%
% This program is free software; you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation; either version 3 of the License, or
% (at your option) any later version.
%
% Written (W) 2005-2009 Gunnar Raetsch, Gabriele Schweikert, Jonas Behr, Soeren Sonnenburg, Alexander Zien, Georg Zeller, Andre Noll, Cheng Soon Ong, Petra Philips
% Copyright (C) 2005-2009 Max Planck Society and Fraunhofer Institute FIRST 
%
function mgene_signal_eval(spf_label_fname, spf_pred_fname)
% mgene_signal_eval(spf_label_fname, spf_pred_fname)

global MGENE_TMP_PATH

if nargin~=2
  first_arg=[] ;
  if nargin>=1, first_arg=spf_label_fname; end ;
  
  if show_usage('mgene_signal_eval', '<label-SPF-file-or-dir>, <prediction-SPF-file-or-dir>', ...
                nargin, first_arg),
    return
  end ;
end ;

if ~isempty(spf_label_fname) && spf_label_fname(end)=='/',
   spf_label_dir = spf_label_fname ;
   spf_label_fname = '' ;
else 
   spf_label_dir = '' ;
end ;	

if ~isempty(spf_pred_fname) && spf_pred_fname(end)=='/',
   spf_pred_dir = spf_pred_fname ;
   spf_pred_fname = '' ;
else 
   spf_pred_dir = '' ;
end ;	

info_fname = tempname(MGENE_TMP_PATH) ;
signal_eval(spf_label_fname, spf_label_dir, spf_pred_fname, spf_pred_dir, info_fname)

system(sprintf('cat %s && rm %s', info_fname, info_fname)) ;

return

function [ROCs, PRCs, signal_score, signal_score_label] = signal_eval(spf_label_fname, spf_label_dir, spf_pred_fname, spf_pred_dir, info_fname)
% [ROCs, PRCs] = signal_eval(fn_label, fn_pred, info_fname)
%

disp('-----------------------------') ;
disp('Step 1: Reading label file...') ;
disp('-----------------------------') ;

spf_label_mat_fname = sprintf('%s/label_spf.mat', spf_label_dir) ;
if fexist(spf_label_mat_fname),
  disp('Reading binary SPF label files') ;
  [chr_label, strand_label, pos_label, score_label, score_names_label, signal_names_label, chr_label_dic, signal_names_label_dic, score_names_label_dic, success]=read_sigpred_bin(spf_label_mat_fname) ;
  assert(success) 
else
  disp('Reading SPF label files') ;
  [chr_label, strand_label, pos_label, score_label, score_names_label, signal_names_label, chr_label_dic, signal_names_label_dic, score_names_label_dic]=read_sigpred(spf_label_fname) ;
end ;

signal_names = unique(signal_names_label_dic) ;

idx_ = strmatch('label', score_names_label_dic, 'exact') ;
idx = find(score_names_label==idx_) ;
if length(idx)~=length(score_names_label),
  warning('dropping %i/%i non-label entries from label file', length(score_names_label)-length(idx), length(score_names_label)) ;
  chr_label = chr_label(idx) ;
  strand_label = strand_label(idx) ;
  pos_label = pos_label(idx) ;
  score_label = score_label(idx) ;
  signal_names_label = signal_names_label(idx) ;
  score_names_label = score_names_label(idx) ;
end 

idx = find(abs(abs(score_label)-1)<=1e-3) ;
if length(idx)~=length(score_label),
  warning('dropping %i/%i entries with label not +/-1 from label file', length(score_label)-length(idx), length(score_label)) ;
  chr_label = chr_label(idx) ;
  strand_label = strand_label(idx) ;
  pos_label = pos_label(idx) ;
  score_label = score_label(idx) ;
  signal_names_label = signal_names_label(idx) ;
  score_names_label = score_names_label(idx) ;
end 
score_label = sign(score_label) ;

fprintf('Labels for the following signals found:\n')
for i=1:length(signal_names_label_dic)
  fprintf(' * %s\n', signal_names_label_dic{i}) ;
end ;
fprintf('Done.\n\n') ;

disp('----------------------------------') ;
disp('Step 2: Reading prediction file...') ;
disp('----------------------------------') ;

spf_pred_mat_fname = sprintf('%s/Conf_cum_spf.mat', spf_pred_dir) ;
if fexist(spf_pred_mat_fname),
  disp('Reading binary SPF prediction files') ;
  [chr, strand, pos, score, signal_names, score_names, chr_dic, signal_names_dic, score_names_dic, success] = read_sigpred_bin(spf_pred_mat_fname) ;
  assert(success) ;
else
  disp('Reading SPF prediction files') ;
  [chr, strand, pos, score, signal_names, score_names, chr_dic, signal_names_dic, score_names_dic] = read_sigpred(spf_pred_fname) ;
end ;

fprintf('Predictions for the following signals found:\n')
for i=1:length(signal_names_dic)
  fprintf(' * %s\n', signal_names_dic{i}) ;
end ;
fprintf('Done.\n\n') ;


eval_signal_names = intersect(signal_names_dic, signal_names_label_dic) ;

if isempty(eval_signal_names),
  error('No predictions can be evaluated (labels and predictions do not match)\n') ;
end ;

disp('---------------------------------') ;
disp('Step 3: Evaluating predictions...') ;
disp('---------------------------------') ;


% determine common chr numbering
[eval_chr, chr_map, chr_label_map] = intersect(lower(chr_dic), lower(chr_label_dic))  ;
chr = -chr ;
for i=1:length(chr_map),
  idx=find(-chr==chr_map(i)) ;
  chr(idx) = i ;
end ;
chr(chr<=0) = nan ;

chr_label = -chr_label ;
for i=1:length(chr_label_map),
  idx=find(-chr_label==chr_label_map(i)) ;
  chr_label(idx) = i ;
end ;
chr_label(chr_label<=0) = nan ;

fi = fopen(info_fname, 'w+') ;
ROCs=[] ;
PRCs=[] ;
for i=1:length(eval_signal_names),
  signal_name = eval_signal_names{i} 
  fprintf(fi, 'Evaluation for "%s" predictions ', signal_name) ;

  idx_label_ = strmatch(signal_name, signal_names_label_dic, 'exact') ;
  idx_label = find(signal_names_label==idx_label_ & ~isnan(chr_label)) ;
  idx_ = strmatch(signal_name, signal_names_dic, 'exact') ;
  idx = find(signal_names == idx_ & ~isnan(chr)) ;

  %id = [chr(idx); double(strand(idx)); pos(idx)]' ;
  %id_label = [chr_label(idx_label); double(strand_label(idx_label)); pos_label(idx_label)]' ;

  id_ = (chr(idx)*100000000+pos(idx)).*(1-2*(strand(idx)=='-')) ;
  id_label_ = (chr_label(idx_label)*100000000+pos_label(idx_label)).*(1-2*(strand_label(idx_label)=='-')) ;

  [tmp, idx_id, idx_id_label] = intersect(id_, id_label_);%, 'rows') ;
  fprintf(fi, 'on %i positions\n', length(idx_id)) ;

  signal_score = score(idx(idx_id)) ;
  signal_score_label = score_label(idx_label(idx_id_label)) ;

  pos_ = pos(idx(idx_id)) ;
  pos_label_ = pos_label(idx_label(idx_id_label)) ;
  assert(isequal(pos_, pos_label_)) ;

  ROC=calcrocscore(signal_score, signal_score_label) 
  PRC=calcrfcscore(signal_score, signal_score_label) 

  fprintf(fi, ' * Area under ROC curve: %1.3f\n', ROC) ;
  fprintf(fi, ' * Area under PRC curve: %1.3f\n\n', PRC) ;

  ROCs = [ROCs ROC] ;
  PRCs = [PRCs PRC] ;
end ;

fclose(fi) ;

fprintf('Done.\n\n') ;

return

function [chr, strand, pos, score, signal_name, score_name, chr_name_dic, signal_name_dic, score_name_dic]=read_sigpred(sigpred_fname)
% [chr, strand, pos, score, signal_name, score_name]=read_sigpred(sigpred_fname)

if fexist([sigpred_fname '.gz']) && ~fexist(sigpred_fname),
  ret = unix(sprintf('gzip -d %s',sigpred_fname)) ;
  assert(ret==0) ;
end ;


chr_name_dic = {} ;
score_name_dic = {} ;
signal_name_dic = {} ;

chr=[] ; 
signal_name = [] ;
score_name = [] ;
strand = '' ;
pos = [] ;
score = [] ;

fd = fopen(sigpred_fname, 'r') ;

rest_text = '' ; part = 0 ; part_offset = 0 ;
line_idx = 0 ;
while ~feof(fd),
  part = part + 1 ;
  text=fread(fd, 100000000, 'uint8=>char') ;
  text=[rest_text text'] ;
  rest_text = '' ;
  if length(text)==0, 
    break ; 
  end ;
  last_idx = find(text==sprintf('\n'), 1, 'last') ;
  if last_idx~=length(text) && ~feof(fd),
    rest_text = text(last_idx+1:end) ;
    text(last_idx:end) = [] ;
  end ;
  text = [text sprintf('\n')] ;
  line_idx = find(text==sprintf('\n')) ;

  if length(line_idx)>2, 
    chr(length(chr)+1 : length(chr) + length(line_idx) -2 ) = nan ; 
    signal_name(length(signal_name)+1:length(signal_name) + length(line_idx)-2) = nan ;
    score_name(length(score_name)+1 : length(score_name) + length(line_idx)-2) = nan ;
    strand(length(strand)+1 : length(strand) + length(line_idx)-2) = ' ' ;
    pos(length(pos)+1 : length(pos) + length(line_idx)-2) = nan ;
    score(length(score)+1 : length(score) + length(line_idx)-2) = nan ;
  end ;

  chr_idx = 0 ;
  signal_name_idx = 0 ;
  score_name_idx = 0 ;
  progress = 0 ;
  for i=1:length(line_idx)-1,
    line=text(line_idx(i)+1:line_idx(i+1)-1) ;
    if isempty(line), 
      chr(i) = nan ;
      score(i) = nan ;
      score_name(i) = nan ;
      signal_name(i) = nan ;
      pos(i) = nan ;
      strand(i) = ' ' ;
      continue; 
    end ;
    if isequal(line(1),'#')
      continue
    end

    elem_idx = find(line==sprintf('\t')) ;
    
    if (i-1)/length(line_idx)>=progress,
      fprintf('\rPart %i: Read line %i/%i (%2.0f%%)     ', part, (i-1), length(line_idx), progress*100);
      progress = progress+0.05 ;
    end ;
    if chr_idx==0 || ~isequal(line(1:elem_idx(1)-1), chr_name_dic{chr_idx}),
      chr_idx = strmatch(line(1:elem_idx(1)-1), chr_name_dic, 'exact') ;
      if isempty(chr_idx),
        chr_name_dic{end+1} = line(1:elem_idx(1)-1) ;
        chr_idx = length(chr_name_dic) ;
      end ;
    end ;
    
    if signal_name_idx==0 || ~isequal(line(elem_idx(1)+1:elem_idx(2)-1), signal_name_dic{signal_name_idx}),
      signal_name_idx = strmatch(line(elem_idx(1)+1:elem_idx(2)-1), signal_name_dic, 'exact') ;
      if isempty(signal_name_idx),
        signal_name_dic{end+1} = line(elem_idx(1)+1:elem_idx(2)-1) ;
        signal_name_idx = length(signal_name_dic) ;
      end ;
    end ;
    
    if score_name_idx==0 || ~isequal(line(elem_idx(2)+1:elem_idx(3)-1), score_name_dic{score_name_idx}),
      score_name_idx = strmatch(line(elem_idx(2)+1:elem_idx(3)-1), score_name_dic, 'exact') ;
      if isempty(score_name_idx),
        score_name_dic{end+1} = line(elem_idx(2)+1:elem_idx(3)-1) ;
        score_name_idx = length(score_name_dic) ;
      end ;
    end ;
    
    chr(i+part_offset) = chr_idx ;
    signal_name(i+part_offset) = signal_name_idx ;
    score_name(i+part_offset) = score_name_idx ;
    strand(i+part_offset) = line(elem_idx(4)+1:elem_idx(5)-1) ;
    pos(i+part_offset) = str2double(line(elem_idx(3)+1:elem_idx(4)-1)) ;
    score(i+part_offset) = str2double(line(elem_idx(5)+1:end)) ;
  end

  part_offset = part_offset + length(line_idx)-1 ;
end ;
fclose(fd) ;

fprintf('\rPart %i: Read line %i/%i (%2.0f%%)              \n', part, ...
        length(line_idx), length(line_idx), 100);
signal_name(isnan(chr)) = [] ;
score_name(isnan(chr)) = [] ;
strand(isnan(chr)) = [] ;
pos(isnan(chr)) = [] ;
score(isnan(chr)) = [] ;
chr(isnan(chr)) = [] ;

assert(all(chr~=0)) ;


return

function [chr, strand, pos, score, signal_name, score_name, chr_name_dic, signal_name_dic, score_name_dic, success]=read_sigpred_bin(sigpred_fname)
% [chr, strand, pos, score, signal_name, score_name, chr_name_dic, signal_name_dic, score_name_dic, success]=read_sigpred_bin(sigpred_fname)

chr_name_dic = {} ;
score_name_dic = {} ;
signal_name_dic = {} ;

chr=[] ; 
signal_name = [] ;
score_name = [] ;
strand = '' ;
pos = [] ;
score = [] ;

if ~fexist(sigpred_fname),
  success=false ;
  return ;
end ;

cnt = 0; last=0;
while ~last,
  try
    SPF=load(sigpred_fname, sprintf('SPF_%i', cnt)) ;
    if ~isfield(SPF, sprintf('SPF_%i', cnt)),
      break
    end ;
  catch
    break ;
  end 
  if ~isempty(SPF)
    eval(sprintf('SPF=SPF.SPF_%i;', cnt)) ;
    cnt=cnt+1 ;
  %else
  %  SPF=load(sigpred_fname) ;
  %  last = 1 ;
  end ;

  if any(ismember(chr_name_dic, SPF.contig_name)),
    contig_idx = find(ismember(chr_name_dic, SPF.contig_name)) ;
  else
    chr_name_dic{end+1} = SPF.contig_name ;
    contig_idx = length(chr_name_dic) ;
  end ;

  if any(ismember(score_name_dic, SPF.score_name)),
    score_idx = find(ismember(score_name_dic, SPF.score_name)) ;
  else
    score_name_dic{end+1} = SPF.score_name ;
    score_idx = length(score_name_dic) ;
  end ;

  if any(ismember(signal_name_dic, SPF.signal_name)),
    signal_idx = find(ismember(signal_name_dic, SPF.signal_name)) ;
  else
    signal_name_dic{end+1} = SPF.signal_name ;
    signal_idx = length(signal_name_dic) ;
  end ;

  chr(end+1:end+length(SPF.pos))=contig_idx ;
  score_name(end+1:end+length(SPF.pos))=score_idx ;
  signal_name(end+1:end+length(SPF.pos))=signal_idx ;
  pos(end+1:end+length(SPF.pos))=SPF.pos ;
  score(end+1:end+length(SPF.pos))=SPF.score ;
  strand(end+1:end+length(SPF.pos))=SPF.strand ;
end ;

success = true ;

return

function [false_alarms,hits] = calcroc(output,LTE)

assert(all(size(output)==size(LTE))) ;

[ld,idx] = sort(output); 

i=0 ;

hits=1-cumsum(LTE(idx)>0)/sum(LTE > 0) ;
false_alarms=1-cumsum(LTE(idx)<0)/sum(LTE < 0) ;

% Da die Kurve nie alle Negativen als positiven zaehlt, hat
% Falsealarms nie 
% den Wert 1. Cumsum zaehlt das erste Element als negatives, diese
% fehlt bei
%  den False_alarms.
% Daher wird vorne an den Vektor noch eine 1 gehaengt

hits = [1 hits ];
false_alarms = [1 false_alarms];

if nargout<2,
  false_alarms = [false_alarms;hits];
end ;

return

function s = calcrocscore(oute,LTE)
% rocscore = calcrocscore(output,LT)
%
% computes the area under the ROC curve

[a,b]=calcroc(oute,LTE) ;

s=0 ;
for i=1:length(a)-1,
  s = s + (a(i)-a(i+1))*(b(i)+b(i+1))/2 ;
end ;


return

function [score,tp,tdr]=calcrfcscore(output,LT)
% score=calcrfcscore(output,LT)
map=(LT==1) ;
[s,idx]=sort(-output) ; s=-s ;
map=map(idx) ;

tp=cumsum(map)/sum(LT==1) ;
tdr=cumsum(map)./[1:length(map)] ;

t=tp(2:end)-tp(1:end-1) ;
score = sum(0.5*[tdr(1:end-1)+tdr(2:end)].*t) ;



return

function x = fexist(filename,gz)
% FEXIST Check if file exists
%        FEXIST('filename') returns:
%        0 if filename does not exist
%        1 if filename exists in the given path

if nargin<2,
  gz=0;
end ;

fd = fopen(filename);
if fd ~= -1
  x = 1;
  fclose(fd);
elseif gz,
  fd = fopen([filename '.gz']);
  if fd~=-1,
    x=1 ;
    fclose(fd) ;
  else
    x=0 ;
  end ;
else
  x = 0;
end

return

function ret=keyboard_allowed()
% ret=keyboard_allowed()
%
% returns 1, if a keyboard command would be allowed

global g_ignore_keyboard
global THIS_IS_A_RPROC_PROCESS  

if isequal(g_ignore_keyboard, 1),
  ret=0 ;
  return ;
end ;

if isequal(THIS_IS_A_RPROC_PROCESS, 1),
  ret=0 ;
  return ;
end ;

ret=1 ;


return

